@@ -1,7 +1,7 @@
 #
 # Config::General::Extended - special Class based on Config::General
 #
-# Copyright (c) 2000-2001 Thomas Linden <tom@daemon.de>.
+# Copyright (c) 2000-2003 Thomas Linden <tom@daemon.de>.
 # All Rights Reserved. Std. disclaimer applies.
 # Artificial License, same as perl itself. Have fun.
 #
@@ -23,7 +23,7 @@ use vars qw(@ISA @EXPORT);
 use strict;
 
 
-$Config::General::Extended::VERSION = "1.8";
+$Config::General::Extended::VERSION = "1.10";
 
 
 sub new {
@@ -40,15 +40,22 @@ sub obj {
   #
   my($this, $key) = @_;
   if (exists $this->{config}->{$key}) {
-    if (!$this->{config}->{$key}) {
-      return $this->SUPER::new( -ExtendedAccess => 1, -ConfigHash => {} ); # empty object!
+    if (!$this->{config}->{$key} || ref($this->{config}->{$key}) ne "HASH") {
+      if ($this->{StrictObjects}) {
+	croak "key \"$key\" does not point to a hash reference!\n";
+      }
+      else {
+	# be cool, create an empty object!
+	return $this->SUPER::new( -ExtendedAccess => 1, -ConfigHash => {}, %{$this->{Params}} );
+      }
     }
     else {
-      return $this->SUPER::new( -ExtendedAccess => 1, -ConfigHash => $this->{config}->{$key} );
+      return $this->SUPER::new( -ExtendedAccess => 1, -ConfigHash => $this->{config}->{$key}, %{$this->{Params}} );
     }
   }
   else {
-    return $this->SUPER::new( -ExtendedAccess => 1, -ConfigHash => $this->{config} );
+    # even return an empty object if $key does not exist
+    return $this->SUPER::new( -ExtendedAccess => 1, -ConfigHash => {}, %{$this->{Params}} );
   }
 }
 
@@ -63,7 +70,17 @@ sub value {
     $this->{config}->{$key} = $value;
   }
   else {
-    return $this->{config}->{$key} if(exists $this->{config}->{$key});
+    if (exists $this->{config}->{$key}) {
+      return $this->{config}->{$key};
+    }
+    else {
+      if ($this->{StrictObjects}) {
+	croak "Key \"$key\" does not exist within current object\n";
+      }
+      else {
+	return "";
+      }
+    }
   }
 }
 
@@ -74,7 +91,17 @@ sub hash {
   # as hash
   #
   my($this, $key) = @_;
-  return %{$this->{config}->{$key}} if(exists $this->{config}->{$key});
+  if (exists $this->{config}->{$key}) {
+    return %{$this->{config}->{$key}};
+  }
+  else {
+    if ($this->{StrictObjects}) {
+      croak "Key \"$key\" does not exist within current object\n";
+    }
+    else {
+      return ();
+    }
+  }
 }
 
 
@@ -84,7 +111,15 @@ sub array {
   # as array
   #
   my($this, $key) = @_;
-  return @{$this->{config}->{$key}} if(exists $this->{config}->{$key});
+  if (exists $this->{config}->{$key}) {
+    return @{$this->{config}->{$key}};
+  }
+  if ($this->{StrictObjects}) {
+      croak "Key \"$key\" does not exist within current object\n";
+    }
+  else {
+    return ();
+  }
 }
 
 
@@ -232,10 +267,10 @@ sub AUTOLOAD {
   }
   elsif (exists $this->{config}->{$key}) {
     if ($this->is_hash($key)) {
-      croak "\"$key\" points to a hash and cannot be automatically accessed\n";
+      croak "Key \"$key\" points to a hash and cannot be automatically accessed\n";
     }
     elsif ($this->is_array($key)) {
-      croak "\"$key\" points to an array and cannot be automatically accessed\n";
+      croak "Key \"$key\" points to an array and cannot be automatically accessed\n";
     }
     else {
       return $this->{config}->{$key};
@@ -243,11 +278,11 @@ sub AUTOLOAD {
   }
   else {
     if ($this->{StrictObjects}) {
-      croak "\"$key\" does not exist within current object\n";
+      croak "Key \"$key\" does not exist within current object\n";
     }
     else {
       # be cool
-      return undef;
+      return "";
     }
   }
 }
@@ -487,7 +522,7 @@ values under the given key will be overwritten.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2000-2002 Thomas Linden
+Copyright (c) 2000-2003 Thomas Linden
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -505,7 +540,7 @@ Thomas Linden <tom@daemon.de>
 
 =head1 VERSION
 
-1.8
+1.10
 
 =cut
 
